using System;
using System.Collections.Generic;
using System.Diagnostics;
using FireStarterModeller.BO;
using FireStarterModeller.Logic;
using Habanero.Base;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.Faces.Base;

namespace FireStarterModeller.UI
{
    /// <summary>
    /// Creates a <see cref="UIGridInfo"/> and its related <see cref="UIGridFilter"/> based on the inputs from the wizard.
    /// </summary>
    public class CreateGridWizardController : WizardController
    {
        private readonly SelectPropertiesWizardStep _selectColumnPropertiesWizardStep;
        //private readonly SelectPropertiesWizardStep _selectFilterPropertiesWizardStep;
        private readonly SelectPropertiesFilterWizardStep _selectFilterPropertiesWizardStep;
        /// <summary>
        /// Gets the <see cref="UIGrid"/> created by this Wizard controller.
        /// </summary>
        public UIGridInfo UIGrid { get; private set; }
        /// <summary>
        /// Gets the <see cref="UIGridFilter"/> Created by this Wizard.
        /// </summary>
        public UIGridFilter GridFilter { get; private set; }

        /// <summary>
        /// Creates a <see cref="CreateGridWizardController"/> of type <see cref="WizardController"/> with the required steps 
        /// for creating a <see cref="UIGridInfo"/> and its related <see cref="UIGridFilter"/>.
        /// </summary>
        public CreateGridWizardController()
        {
            _selectColumnPropertiesWizardStep = new SelectPropertiesWizardStep();
            //_selectFilterPropertiesWizardStep = new SelectPropertiesWizardStep();
            _selectFilterPropertiesWizardStep = new SelectPropertiesFilterWizardStep();
            SelectFilterPropertiesWizardStep.ClassSelectorControl.Visible = false;
            this.AddStep(_selectColumnPropertiesWizardStep);
            this.AddStep(SelectFilterPropertiesWizardStep);
            _selectColumnPropertiesWizardStep.ClassSelectorControl.BusinessObjectSelected += ClassSelectorControl_BusinessObjectSelected;
            _selectColumnPropertiesWizardStep.Label.Text = "1. Please select an entity and the properties to show in the grid.";
        }

        private void ClassSelectorControl_BusinessObjectSelected(object sender, BOEventArgs e)
        {
            try
            {
                DMClass dmClass = (DMClass)e.BusinessObject;
                SelectFilterPropertiesWizardStep.SelectedClass = dmClass;
                if (dmClass != null)
                {
                    //CheckClassHasUIViews(dmClass);
                    SelectFilterPropertiesWizardStep.Label.Text = "2. Select filters for the grid definition of the entity '" +
                                                                   dmClass.ToString() + "'.";
                }
            }
            catch (Exception ex)
            {
                GlobalRegistry.UIExceptionNotifier.Notify(ex, "", "Error ");
            }
        }
        /// <summary>
        /// The DMClass that the <see cref="UIGridInfo"/> and its related <see cref="UIGridFilter"/> are created for.
        /// </summary>
        public DMClass Class
        {
            get { return _selectColumnPropertiesWizardStep.SelectedClass; }
            set
            {
                //CheckClassHasUIViews(value);
                _selectColumnPropertiesWizardStep.SelectedClass = value;
                SelectFilterPropertiesWizardStep.SelectedClass = value;
            }
        }

        //private static void CheckClassHasUIViews(DMClass dmClass)
        //{

        //    if (dmClass != null && dmClass.UIViews.Count > 0)   
        //    {
        //        throw new UserException("You cannot create a grid definition for the entity '" + dmClass.ToString() +
        //                                "' as it already has a UIView defined.");
        //    }
        //}
        /// <summary>
        /// The <see cref="DMSolution"/> that the <see cref="UIGridInfo"/> and its related <see cref="UIGridFilter"/> are being created for.
        /// This is used to fill the <see cref="DMClass"/>es that are available.
        /// </summary>
        public DMSolution Solution
        {
            get { return _selectColumnPropertiesWizardStep.Solution; }
            set { _selectColumnPropertiesWizardStep.Solution = value; }
        }

        /// <summary>
        /// Returns the Wizard step that is used to Select the properties that will be used for the filter.
        /// </summary>
        public SelectPropertiesFilterWizardStep SelectFilterPropertiesWizardStep
        {
            get { return _selectFilterPropertiesWizardStep; }
        }

        /// <summary>
        /// Returns the Wizard step that is used to Select the properties that will be used for the filter.
        /// </summary>
        public SelectPropertiesWizardStep SelectPropertiesWizardStep
        {
            get { return _selectColumnPropertiesWizardStep; }
        }

        ///<summary>
        /// Method that is to be run when the Wizard is finished. This method should do all persistance that is required.
        ///</summary>
        public override void Finish()
        {
            try
            {

                BusinessObjectCollection<UIView> uiViews = this.Class.UIViews;
                //for (int i = uiViews.Count - 1; i >= 0; i--)
                //{
                //    UIView view = uiViews[i];
                //    view.MarkForDelete();
                //    view.Save();
                //}
                UIGridCreator uiGridCreator = new UIGridCreator();
                UIGrid = uiGridCreator.CreateGrid(this.Class, _selectColumnPropertiesWizardStep.PropertiesSelectorControl.SelectedOptions);
                UIGridFilterCreator gridFilterCreator = new UIGridFilterCreator(Solution);
                IList<string> selectedFilerOptions = SelectFilterPropertiesWizardStep.PropertiesSelectorControl.SelectedOptions;
                if (selectedFilerOptions.Count > 0)
                {
                    GridFilter = gridFilterCreator.CreateUIGridFilter(selectedFilerOptions);
                    UIGrid.UIGridFilter = GridFilter;
                }
                UIView uiView = uiViews.CreateBusinessObject();
                uiView.UIGridInfo = UIGrid;

                UIBOEditorCreator boEditorCreator = new UIBOEditorCreator();
                UIFormInfo uiFormInfo = boEditorCreator.CreateBOEditor(this.Class);
                uiView.UIFormInfo = uiFormInfo;
                uiView.ViewName = this.Class.GetUniqueViewNameWithSuffix(uiView.ViewName);
                uiView.Save();
                base.Finish();
            }
            catch (Exception ex)
            {
                GlobalRegistry.UIExceptionNotifier.Notify(ex, "", "Error ");
            }
        }

        ///<summary>
        /// Checks if the Wizard can proceed to the next step. Calls through to the CanMoveOn method of the current IWizardStep. 
        /// You can move on from Select Grid Column Properties as long as you have selects some Grid Properties.         
        ///</summary>
        ///
        ///<param name="message">Describes why the Wizard cannot move on. Only applicable if CanMoveOn returns false.</param>
        ///<returns>
        ///True if moving to the next step is allowed.
        ///</returns>
        ///
        public override bool CanMoveOn(out string message)
        {
            base.CanMoveOn(out message);
            MultiSelectorModel<string> multiSelectorModel = _selectColumnPropertiesWizardStep.PropertiesSelectorControl.Model;
            if (CurrentStep == 0)
            {
                //CheckClassHasUIViews(this.SubClass);
                if (multiSelectorModel.SelectedOptions.Count == 0)
                {
                    message += Environment.NewLine + "You must select properties to show in the grid before you can move on";
                }
            }
            if (CurrentStep == 0)
            {
                SelectFilterPropertiesWizardStep.PropertiesSelectorControl.AllOptions = multiSelectorModel.SelectedOptions;
            }
            return message.Length <= 0;
        }
    }
}