using System;
using System.Data;
using System.Data.Common;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.DB;

namespace Firestarter.DB.Schema.Logic
{
    public class DatabaseReverseEngineer
    {
        private readonly IDBSchemaExtractor _dbSchemaExtractor;
        private DataTable _columnsDataTable;
        private DataTable _primaryKeyColumnsDataTable;
        private DataTable _uniqueKeyColumnsDataTable;
        private DataTable _foreignKeyColumnsDataTable;

        public DatabaseReverseEngineer(IDBSchemaExtractor dbSchemaExtractor)
        {
            _dbSchemaExtractor = dbSchemaExtractor;
        }

        /// <summary>
        /// Gets and sets the type converter that converts custom database types
        /// to a set of standards provided by default or by the user
        /// </summary>
        public DBTypeConverter DBTypeConverter { get; set; }

        private DataTable GetTablesDataTable()
        {
            return _dbSchemaExtractor.GetTablesDataTable();
        }

        public DBDatabase CreateDatabaseInfo(string databaseName)
        {
            //TODO: Test This
            DataTable dataTable = _dbSchemaExtractor.GetDatabasesDataTable(databaseName);
            foreach (DataRow dataRow in dataTable.Rows)
            {
                return new DBDatabase
                {
                    Catalog = ((string) DBUtilities.ConvertDBNullToNull(dataRow[DBSchemaTemplate.Naming.CatalogName])),
                    Schema = ((string) DBUtilities.ConvertDBNullToNull(dataRow[DBSchemaTemplate.Naming.SchemaName])),
                    DatabaseName = databaseName
                };
            }
            return null;
        }

        public void PopulateDatabaseInfo(DBDatabase dbDatabase)
        {
            if (dbDatabase == null) throw new ArgumentNullException("dbDatabase");
            //IDatabaseConnection databaseConnection = dbDatabase.DatabaseConnection;
            //if (databaseConnection == null)
            //{
            //    throw new HabaneroArgumentException
            //        ("dbDatabase", "The dbDatabase has not been set up with a database connection");
            //}
            ////DataTable tableList = GetTableDataTable(databaseConnection.GetConnection());
            PopulateTables(dbDatabase);
            //For Performance the DBReverseEngineering loads all columns etc and then
            // matches them to the Table later.
            _columnsDataTable = _dbSchemaExtractor.GetColumnsDataTable(null);
            _primaryKeyColumnsDataTable = _dbSchemaExtractor.GetPrimaryKeyColumnsDataTable(null);
            _uniqueKeyColumnsDataTable = _dbSchemaExtractor.GetUniqueKeyColumnsDataTable(null);
            _foreignKeyColumnsDataTable = _dbSchemaExtractor.GetForeignKeyColumnsDataTable(null, null);

            // foreach gets collection modification error

            foreach (DBTable dbTable in dbDatabase.Tables.ToArray())
            {
                PopulateTableInfo(dbTable);
            }
            //The Relationships must be done last so as to ensure that all the 
            //Table with their properties and Unique constraints (Alternate Keys) are loaded.
            foreach (DBTable dbTable in dbDatabase.Tables.ToArray())
            {
                PopulateForeignKeys(dbTable, _foreignKeyColumnsDataTable);
            }
        }

        public void PopulateTables(DBDatabase dbDatabase)
        {
            DataTable tableList = GetTablesDataTable();
            foreach (DataRow tableInfoDataRow in tableList.Rows)
            {
                DBTable dbTable = CreateTableInfo(tableInfoDataRow);
                DBTable existingDBTable = dbDatabase.GetTableInfo(dbTable.TableNameDB);
                if (existingDBTable == null)
                {
                    dbDatabase.Tables.Add(dbTable);
                }
            }
        }

        public void PopulateTableInfo(DBTable dbTable)
        {
            if (dbTable == null) throw new ArgumentNullException("dbTable");
            if (dbTable.Database == null)
                throw new NullReferenceException
                    ("You cannot generate Table Info since it does not have a reference to DBDatabase");

            //IDatabaseConnection databaseConnection = dbTable.Database.DatabaseConnection;
            //if (databaseConnection == null)
            //{
            //    throw new HabaneroArgumentException
            //        ("dbTable", "The dbTable.Database has not been set up with a database connection");
            //}
            PopulateColumns(dbTable, _columnsDataTable);
            PopulatePrimaryKeyInfos(dbTable, _primaryKeyColumnsDataTable);
            PopulateUniqueKeyInfos(dbTable, _uniqueKeyColumnsDataTable);
        }

        public delegate void ProcessDataRowDelegate(DataRow dataRow, DBTable dbTable);
        private static void UpdateTableInfo(DataTable dataTable, DBTable dbTable, ProcessDataRowDelegate rowProcessor)
        {
            DataTable dataTableSubSet = GetDataTableSubSet(dataTable, dbTable.TableNameDB);
            foreach (DataRow columnInfoDataRow in dataTableSubSet.Rows)
            {
                rowProcessor(columnInfoDataRow, dbTable);
            }
        }

        private static DataTable GetDataTableSubSet(DataTable dataTable, string tableName)
        {
            DataView dataView = new DataView(dataTable);
            string rowFilter;
            if (tableName == null)
            {
                rowFilter = string.Format("Isnull({0},'Null Column') = 'Null Column'", DBSchemaTemplate.Naming.TableName);
            }
            else
            {
                rowFilter = string.Format("{0} = '{1}'", DBSchemaTemplate.Naming.TableName, tableName);
            }
            dataView.RowFilter = rowFilter;
            dataTable = dataView.ToTable();
            return dataTable;
        }

        public void PopulateColumns(DBTable dbTable, DataTable dataTable)
        {
            UpdateTableInfo(dataTable, dbTable, CreateColumnInfo);
        }

        public void PopulatePrimaryKeyInfos(DBTable dbTable, DataTable dataTable)
        {
            UpdateTableInfo(dataTable, dbTable, CreatePrimaryKeyInfo);
        }

        public void PopulateUniqueKeyInfos(DBTable dbTable, DataTable dataTable)
        {
            UpdateTableInfo(dataTable, dbTable, CreateAlternateKeyInfo);
        }

        public void PopulateForeignKeys(DBTable dbTable, DataTable dataTable)
        {
            UpdateTableInfo(dataTable, dbTable, CreateForeignKeyInfo);
        }

        private static DBTable CreateTableInfo(DataRow tableInfoDataRow)
        {
            return new DBTable
            {
                TableNameDB = ((string)DBUtilities.ConvertDBNullToNull(tableInfoDataRow[DBSchemaTemplate.Naming.TableName])),
                TableType = ((string)DBUtilities.ConvertDBNullToNull(tableInfoDataRow[DBSchemaTemplate.Naming.TableType])),
            };
        }

        private void CreateColumnInfo(DataRow columnInfoDataRow, DBTable dbTable)
        {
            //If column exist get existing column
            //  Check if column has changed.
            //  If changed update and mark as changed
            //Else Create new column
            //   mark new column as new
            //Check if column is deleted
            string dataType = DBUtilities.ConvertDBToString(columnInfoDataRow[DBSchemaTemplate.Naming.DataType]);
            string convertedDataType = ConvertDataType(dataType, DBTypeConverter);
            //TODO brett 25 Feb 2010: IS_Nullable does ot follow constant namimng rule
            DBColumn dbColumn = new DBColumn
            {
                ColumnName = DBUtilities.ConvertDBToString(columnInfoDataRow[DBSchemaTemplate.Naming.ColumnName]),
                DataType = convertedDataType,
                Nullable = DBUtilities.ConvertToBoolean(columnInfoDataRow["IS_NULLABLE"]),
                DefaultValue = DBUtilities.ConvertDBToString(columnInfoDataRow[DBSchemaTemplate.Naming.ColumnDefault]),
                DataLength = GetCharDataLength(columnInfoDataRow),
                OrdinalPosition = DBUtilities.ConvertDBToInt(columnInfoDataRow[DBSchemaTemplate.Naming.OrdinalPosition])
            };
            dbTable.Columns.Add(dbColumn);
        }

        internal static string ConvertDataType(string dataType, DBTypeConverter dbTypeConverter)
        {
            if (dbTypeConverter == null) return dataType;
            return dbTypeConverter.Convert(dataType);
        }

        internal static void CreatePrimaryKeyInfo(DataRow columnInfoDataRow, DBTable dbTable)
        {
            if (dbTable.PrimaryKeyDB == null) dbTable.PrimaryKeyDB = new DBPrimaryKey();
            string columnInfoName = (string)columnInfoDataRow[DBSchemaTemplate.Naming.ColumnName];
            DBColumn dbColumn = dbTable.GetColumnInfo(columnInfoName);

            CheckColumnExists(dbTable, dbColumn);
            dbTable.PrimaryKeyDB.PrimaryKeyColumns.Add(new DBPrimaryKeyColumn { Column = dbColumn });
        }

        private static void CheckColumnExists(DBTable dbTable, DBColumn dbColumn)
        {
            if (dbColumn == null)
            {
                throw new HabaneroApplicationException
                    ("You cannot get the key for the table " + dbTable.TableNameDB
                     + " since the tables columns have not been loaded yet");
            }
        }

        private static void CreateAlternateKeyInfo(DataRow alternateKeyRow, DBTable dbTable)
        {
            string indexName = (string) alternateKeyRow[DBSchemaTemplate.Naming.KeyName];
            string columnInfoName = (string) alternateKeyRow[DBSchemaTemplate.Naming.ColumnName];
            DBColumn dbColumn = dbTable.GetColumnInfo(columnInfoName);

            CheckColumnExists(dbTable, dbColumn);

            DBKey existingDBKey = dbTable.GetKeyInfo(indexName);
            if (existingDBKey == null)
            {
                DBKey dbKey = new DBKey();
                dbKey.KeyColumns.Add(new DBKeyColumn { Column = dbColumn });
                dbKey.KeyName = indexName;
                dbTable.Keys.Add(dbKey);
            }
            else
            {
                existingDBKey.KeyColumns.Add(new DBKeyColumn { Column = dbColumn });
            }
        }

        private static void CreateForeignKeyInfo(DataRow row, DBTable dbTable)
        {
            string foreignKeyName = (string) row[DBSchemaTemplate.Naming.KeyName];
            DBForeignKey foundForeignKey = dbTable.GetForeignKeyInfo(foreignKeyName);
            
            string referencedTableName = (string) row[DBSchemaTemplate.Naming.ReferencedTableName];
            DBTable referencedDBTable = dbTable.Database.GetTableInfo(referencedTableName);

            CheckReferencedTableExists(dbTable, foreignKeyName, referencedTableName, referencedDBTable);
            if (foundForeignKey == null)
            {
                foundForeignKey = new DBForeignKey {ForeignKeyName = foreignKeyName, RelatedTable = referencedDBTable};
                dbTable.ForeignKeys.Add(foundForeignKey);
            }
            
            DBForeignKeyColumn dbForeignKeyColumn = new DBForeignKeyColumn();
            string columnInfoName = (string)row[DBSchemaTemplate.Naming.ColumnName];
            DBColumn foundColumnInfo = dbTable.GetColumnInfo(columnInfoName);
            dbForeignKeyColumn.ParentColumn = foundColumnInfo;
            string referencedColumnName = (string)row[DBSchemaTemplate.Naming.ReferencedColumnName];
            DBColumn relatedDBColumn = referencedDBTable.GetColumnInfo(referencedColumnName);
            dbForeignKeyColumn.RelatedColumn = relatedDBColumn;
            foundForeignKey.ForeignKeyColumns.Add(dbForeignKeyColumn);
        }

        private static void CheckReferencedTableExists
            (DBTable dbTable, string foreignKeyName, string referencedTableName, DBTable referencedDBTable)
        {
            if (referencedDBTable == null)
            {
                throw new HabaneroApplicationException
                    (string.Format
                         ("You cannot Update the Foreign Key {0} for table {1} since the referenced table"
                          + " {2} does not exist in the schema", foreignKeyName, dbTable.TableNameDB,
                          referencedTableName));
            }
        }

        public static int GetCharDataLength(DataRow row)
        {
            long dataLength = Convert.ToInt64(DBUtilities.ConvertDBNullToNull(row[DBSchemaTemplate.Naming.CharacterMaximumLength]));
            if (dataLength == 0)
            {
                string[] parts = GetColumnTypeParts(row, DBSchemaTemplate.Naming.DataType);
                if (parts.Length <= 1) parts = GetColumnTypeParts(row, "ColumnType");
                if (parts.Length > 1)
                {
                    string lengthInfo = parts[1];
                    lengthInfo = lengthInfo.Split(new[] {','})[0];
                    dataLength = Convert.ToInt64(lengthInfo);
                }
            }
            int intDataLength = dataLength > int.MaxValue ? int.MaxValue : Convert.ToInt32(dataLength);
            return intDataLength;
        }

        private static string[] GetColumnTypeParts(DataRow row, string columnName)
        {
            string[] parts = new string[] {};
            if (row.Table.Columns.Contains(columnName))
            {
                string columnType = Convert.ToString(DBUtilities.ConvertDBNullToNull(row[columnName]));
                parts = columnType.Split(new[] {'(', ')'}, StringSplitOptions.RemoveEmptyEntries);
            }
            return parts;
        }
    }
}
