using System;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    [TestFixture]
    public class TestDBForeignKey : TestBase
    {
        [Test]
        public void TestGetForeignKeyColumnInfo_NullWhenNoColumnsExist()
        {
            //---------------Set up test pack-------------------
            DBForeignKey dbForeignKey = new DBForeignKey();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, dbForeignKey.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            string nonExistentColumn = TestUtils.GetRandomString();
            DBForeignKeyColumn dbForeignKeyColumn = dbForeignKey.GetForeignKeyColumnInfo(nonExistentColumn);
            //---------------Test Result -----------------------
            Assert.IsNull(dbForeignKeyColumn);
        }

        [Test]
        public void TestGetForeignKeyColumnInfo_NullIfNotFound()
        {
            //---------------Set up test pack-------------------
            DBForeignKey dbForeignKey = new DBForeignKey();

            DBForeignKeyColumn firstDBForeignKeyColumn = new DBForeignKeyColumn();
            firstDBForeignKeyColumn.ParentColumn = new DBColumn("TestColumn1");
            dbForeignKey.ForeignKeyColumns.Add(firstDBForeignKeyColumn);

            DBForeignKeyColumn secondDBForeignKeyColumn = new DBForeignKeyColumn();
            secondDBForeignKeyColumn.ParentColumn = new DBColumn("TestColumn2");
            dbForeignKey.ForeignKeyColumns.Add(secondDBForeignKeyColumn);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string nonExistentColumn = TestUtils.GetRandomString();
            DBForeignKeyColumn dbForeignKeyColumn = dbForeignKey.GetForeignKeyColumnInfo(nonExistentColumn);
            //---------------Test Result -----------------------
            Assert.IsNull(dbForeignKeyColumn);
        }

        [Test]
        public void TestGetForeignKeyColumnInfo_FindsExistingOne()
        {
            //---------------Set up test pack-------------------
            DBForeignKey dbForeignKey = new DBForeignKey();

            DBForeignKeyColumn firstDBForeignKeyColumn = new DBForeignKeyColumn();
            firstDBForeignKeyColumn.ParentColumn = new DBColumn("TestColumn1");
            dbForeignKey.ForeignKeyColumns.Add(firstDBForeignKeyColumn);

            DBForeignKeyColumn secondDBForeignKeyColumn = new DBForeignKeyColumn();
            const string columnName = "TestColumn2";
            secondDBForeignKeyColumn.ParentColumn = new DBColumn(columnName);
            dbForeignKey.ForeignKeyColumns.Add(secondDBForeignKeyColumn);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            DBForeignKeyColumn dbForeignKeyColumn = dbForeignKey.GetForeignKeyColumnInfo(columnName);
            //---------------Test Result -----------------------
            Assert.IsNotNull(dbForeignKeyColumn);
        }

        [Test]
        public void Test_AddNullColumnInfo()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            try
            {
                key.AddColumnInfo(null, null);
                Assert.Fail("expected Err");
            }
                //---------------Test Result -----------------------
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("Value cannot be null", ex.Message);
                StringAssert.Contains("parentColumn", ex.ParamName);
            }
        }

        [Test]
        public void Test_AddNullParentColumnInfo()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            try
            {
                key.AddColumnInfo(null, new DBColumn());
                Assert.Fail("expected Err");
            }
                //---------------Test Result -----------------------
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("Value cannot be null", ex.Message);
                StringAssert.Contains("parentColumn", ex.ParamName);
            }
        }

        [Test]
        public void Test_AddNullRelatedColumnInfo()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            DBColumn dbColumn = new DBColumn();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            DBForeignKeyColumn keyColumn = key.AddColumnInfo(dbColumn, null);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, key.ForeignKeyColumns.Count);
            Assert.IsNotNull(keyColumn);
            Assert.AreSame(dbColumn, keyColumn.ParentColumn);
            Assert.IsNull(keyColumn.RelatedColumn);
        }

        [Test]
        public void Test_AddColumnInfo()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            DBColumn parentDBColumn = new DBColumn("TestColumn1");
            DBColumn relatedDBColumn = new DBColumn("TestRelatedColumnInfo");
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            DBForeignKeyColumn keyColumn = key.AddColumnInfo(parentDBColumn, relatedDBColumn);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, key.ForeignKeyColumns.Count);
            Assert.IsNotNull(keyColumn);
            Assert.AreSame(parentDBColumn, keyColumn.ParentColumn);
            Assert.AreSame(relatedDBColumn, keyColumn.RelatedColumn);
        }

        [Test]
        public void Test_AddSecondColumnInfo()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            DBColumn firstDBColumn = new DBColumn("TestColumn1");
            key.ForeignKeyColumns.Add(new DBForeignKeyColumn {ParentColumn = firstDBColumn});
            DBColumn seondDBColumn = new DBColumn("TestColumn2");
            DBColumn relatedDBColumn = new DBColumn("TestRelatedColumnInfo");

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            DBForeignKeyColumn keyColumn = key.AddColumnInfo(seondDBColumn, relatedDBColumn);
            //---------------Test Result -----------------------
            Assert.AreEqual(2, key.ForeignKeyColumns.Count);
            Assert.IsNotNull(keyColumn);
            Assert.AreSame(seondDBColumn, keyColumn.ParentColumn);
            Assert.AreSame(relatedDBColumn, keyColumn.RelatedColumn);
        }

        [Test]
        public void Test_AddColumnInfoTwice()
        {
            //---------------Set up test pack-------------------
            DBForeignKey key = new DBForeignKey();
            DBColumn firstDBColumn = new DBColumn("TestColumn1");
            key.ForeignKeyColumns.Add(new DBForeignKeyColumn {ParentColumn = firstDBColumn});
            DBColumn relatedDBColumn = new DBColumn("TestRelatedColumnInfo");

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, key.ForeignKeyColumns.Count);
            //---------------Execute Test ----------------------
            DBForeignKeyColumn keyColumn = key.AddColumnInfo(firstDBColumn, relatedDBColumn);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, key.ForeignKeyColumns.Count);
            Assert.IsNotNull(keyColumn);
            Assert.AreSame(firstDBColumn, keyColumn.ParentColumn);
        }
    }
}