using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Firestarter.DB.Schema.Logic;
using FireStarterModeller.UI;
using Habanero.Base;
using Habanero.BO;
using Habanero.Faces.Base;
using Habanero.Faces.Win;
using TreeViewCancelEventArgs=Habanero.Faces.Base.TreeViewCancelEventArgs;
using TreeViewController=Habanero.Faces.Base.TreeViewController;
using TreeViewEventArgs=Habanero.Faces.Base.TreeViewEventArgs;

namespace Firestarter.DB.Schema.UI
{
    public class DatabaseInfoChangesControl : UserControlWin, IFormControl
    {
        private TreeViewWin _treeView;
        private FilteredTreeViewController _treeViewController;
        private IPanel _filterPanel;
        private ICheckBox _unchangedCheckbox;
        private ICheckBox _addedCheckbox;
        private ICheckBox _removedCheckbox;
        private ICheckBox _upadtedCheckbox;
        private ImageList _imageList;

        public DatabaseInfoChangesControl()
        {
            SetupTreeView();
            SetupFilterPanel();

            BorderLayoutManagerWin borderLayoutManager = new BorderLayoutManagerWin(this, GlobalUIRegistry.ControlFactory);
            borderLayoutManager.AddControl(_filterPanel, BorderLayoutManager.Position.North);
            borderLayoutManager.AddControl(_treeView, BorderLayoutManager.Position.Centre);
        }

        private void SetupFilterPanel()
        {
            IControlFactory factory = GlobalUIRegistry.ControlFactory;
            _filterPanel = factory.CreatePanel();
            FlowLayoutManager flowLayoutManager = new FlowLayoutManager(_filterPanel, factory);
            _unchangedCheckbox = CreateAndAddFilterCheckbox(flowLayoutManager, factory, "Unchanged");
            _addedCheckbox = CreateAndAddFilterCheckbox(flowLayoutManager, factory, "Added");
            _removedCheckbox = CreateAndAddFilterCheckbox(flowLayoutManager, factory, "Removed");
            _upadtedCheckbox = CreateAndAddFilterCheckbox(flowLayoutManager, factory, "Updated");
            _filterPanel.Height = _unchangedCheckbox.Height + 5;
        }

        private ICheckBox CreateAndAddFilterCheckbox(FlowLayoutManager flowLayoutManager, IControlFactory factory, string text)
        {
            ICheckBox checkbox = factory.CreateCheckBox(true);
            checkbox.Text = text;
            checkbox.CheckedChanged += (sender, e) => UpdateFilter();
            flowLayoutManager.AddControl(checkbox);
            return checkbox;
        }

        private void UpdateFilter()
        {
            IBusinessObject rootNodeBusinessObject = _treeViewController.RootNodeBusinessObject;
            Dictionary<IBusinessObject, ChangeType> allPendingChangeTypes = ChangeTypeTreeInspector.GetAllPendingChangeTypes(rootNodeBusinessObject);
            _treeViewController.ApplyFilter(delegate(IBusinessObject bo)
            {
                IMergableBo mergableBo = bo as IMergableBo;
                if (mergableBo == null) return true;
                ChangeType pendingChangeTypes = allPendingChangeTypes[mergableBo];
                if (_unchangedCheckbox.Checked && HasChangeType(pendingChangeTypes, ChangeType.None)) return true;
                if (_addedCheckbox.Checked && HasChangeType(pendingChangeTypes, ChangeType.Add)) return true;
                if (_removedCheckbox.Checked && HasChangeType(pendingChangeTypes, ChangeType.Delete)) return true;
                if (_upadtedCheckbox.Checked && HasChangeType(pendingChangeTypes, ChangeType.Update)) return true;
                return false;
            });
        }

        private static bool HasChangeType(ChangeType changeType, ChangeType checkChangeType)
        {
            return ((changeType & checkChangeType) == checkChangeType);
        }

        private void SetupTreeView()
        {
            _treeView = new TreeViewWin();
            _treeView.CheckBoxes = true;
            _imageList = new ImageList();
            _imageList.Images.Add("None", Icons.None);
            _imageList.Images.Add("Add", Icons.Add);
            _imageList.Images.Add("Delete", Icons.Delete);
            _imageList.Images.Add("Change", Icons.Change);
            _treeView.ImageList = _imageList;
            _treeViewController = new FilteredTreeViewController(_treeView);

            _treeView.BeforeCheck += delegate(object sender, TreeViewCancelEventArgs e)
            {
                if (e.Node.Tag == null) return;
                if (e.Node.Tag is IMergableBo) return;
                e.Cancel = true;
            };
            _treeView.AfterCheck += delegate(object sender, TreeViewEventArgs e)
            {
                ITreeNode node = e.Node;
                IMergableBo mergableBo = node.Tag as IMergableBo;
                if (mergableBo != null) mergableBo.PendingChangeApproved = node.Checked;
            };

            _treeViewController.SetupNodeWithBusinessObject = delegate(ITreeNode node, IBusinessObject businessObject)
            {
                IMergableBo mergableBo = businessObject as IMergableBo;
                if (mergableBo == null) return;
                node.Checked = mergableBo.PendingChangeApproved;
                SetNodeImage(mergableBo.PendingChangeType, node);
                SetNodeText(node, businessObject);
            };

            _treeViewController.SetupNodeWithRelationship = delegate(ITreeNode node, IRelationship relationship)
            {
                ChangeType pendingChangeType = ChangeType.None;
                if (relationship is IMultipleRelationship)
                {
                    IMultipleRelationship multipleRelationship = (IMultipleRelationship) relationship;
                    IEnumerable<IMergableBo> mergableBos = multipleRelationship.BusinessObjectCollection.OfType<IMergableBo>().Where(mergableBo => mergableBo.PendingChangeType != ChangeType.None);
                    foreach (IMergableBo mergableBo in mergableBos)
                    {
                        if (pendingChangeType == ChangeType.None)
                        {
                            switch (mergableBo.PendingChangeType)
                            {
                                case ChangeType.Add:
                                    pendingChangeType = ChangeType.Add;
                                    continue;
                                case ChangeType.Delete:
                                    pendingChangeType = ChangeType.Delete;
                                    continue;
                            }
                        }
                        pendingChangeType = ChangeType.Update;
                    }
                }
                else if (relationship is ISingleRelationship)
                {
                    ISingleRelationship singleRelationship = (ISingleRelationship) relationship;
                    IMergableBo mergableBo = singleRelationship.GetRelatedObject() as IMergableBo;
                    if (mergableBo != null)
                    {
                        pendingChangeType = mergableBo.PendingChangeType;
                    }
                }
                node.Checked = false;
                SetNodeImage(pendingChangeType, node);
            };
        }

        private void SetNodeText(ITreeNode node, IBusinessObject businessObject)
        {
            if (DoSetNodeText<DBColumn>(node, businessObject, bo => bo.ColumnName )) return;
            if (DoSetNodeText<DBPrimaryKey>(node, businessObject, bo => "Primary" )) return;
            if (DoSetNodeText<DBPrimaryKeyColumn>(node, businessObject, bo => bo.ColumnName )) return;
            if (DoSetNodeText<DBKey>(node, businessObject, bo => bo.KeyName )) return;
            if (DoSetNodeText<DBKeyColumn>(node, businessObject, bo => bo.ColumnName )) return;
            if (DoSetNodeText<DBForeignKey>(node, businessObject, bo => string.Format("{0} ({1})"
                    , bo.ForeignKeyName, bo.RelatedTableName) )) return;
            if (DoSetNodeText<DBForeignKeyColumn>(node, businessObject, bo => string.Format("{0} -> {1}",
                bo.ParentColumn != null ? bo.ParentColumnName : "(Unspecified)", 
                bo.RelatedColumn != null ? bo.RelatedColumnName : "(Unspecified)" ) )) return;
        }

        private bool DoSetNodeText<TBusinessObject>(ITreeNode node, IBusinessObject businessObject, 
            Func<TBusinessObject, string> getTextFunc)
        {
            if (businessObject is TBusinessObject)
            {
                node.Text = getTextFunc((TBusinessObject)businessObject);
                return true;
            }
            return false;
        }

        public DBDatabase Database
        {
            get { return (DBDatabase)_treeViewController.RootNodeBusinessObject; }
            set { _treeViewController.LoadTreeView(value, 7); }
        }

        public TreeViewController TreeViewController
        {
            get { return _treeViewController; }
        }

        public IPanel FilterPanel
        {
            get { return _filterPanel; }
        }

        private void SetNodeImage(ChangeType pendingChangeType, ITreeNode node)
        {
            string imageName;
            switch (pendingChangeType)
            {
                case ChangeType.None:
                    imageName = "None";
                    node.Checked = false;
                    break;
                case ChangeType.Add:
                    imageName = "Add";
                    break;
                case ChangeType.Delete:
                    imageName = "Delete";
                    break;
                case ChangeType.Update:
                    imageName = "Change";
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
            node.SelectedImageIndex = GetImageIndex(imageName);
            node.ImageIndex = GetImageIndex(imageName);
            //TODO Mark 20 Mar 2009: Surely the logic from GetImageIndex below can be used to create implementations for the missing VWG methods commented out below.
            //node.SelectedImageKey = imageName;
            //node.ImageKey = imageName;
        }

        private int GetImageIndex(string imageName)
        {
            return _imageList.Images.IndexOfKey(imageName);
        }

        public void SetForm(IFormHabanero form)
        {
        }
    }
}
