using System;
using Firestarter.Test.Base;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    [TestFixture]
    public class TestDMRelationshipSingleToSingleAKRule
    {
        [SetUp]
        public void SetupTest()
        {
            //Runs every time that any testmethod is executed
            TestUtilsDomain.SetupTest();
            BORegistry.DataAccessor = new DataAccessorInMemory();
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //Code that is executed before any test is run in this class. If multiple tests
            // are executed then it will still only be called once.
            TestUtilsDomain.SetupFixture();
        }

        [TearDown]
        public void TearDownTest()
        {
            //runs every time any testmethod is complete
        }

        #region DMRelationshipSingleToSingleAKRule

        //Both sides cannot be only an AK, Single-to-single must be PK to AK(FK)

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_ForSingleRelationship()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsDMClass.CreateUnsavedDefaultDMClass();
            DMRelationship dmRelationship = dmClass.OwnerRelationships.CreateBusinessObject();
            dmRelationship.Cardinality = Cardinality.Single;
            //---------------Assert Precondition----------------
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            //---------------Execute Test ----------------------
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Test Result -----------------------
            Assert.IsNotNull(rule);
            Assert.AreEqual("DMRelationshipSingleToSingleAK", rule.Name);
            Assert.AreEqual(ErrorLevel.Warning, rule.ErrorLevel);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_ForSingleRelationship_NullClassDM()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsDMClass.CreateUnsavedDefaultDMClass();
            DMRelationship dmRelationship = dmClass.OwnerRelationships.CreateBusinessObject();
            dmRelationship.Cardinality = Cardinality.Single;
            dmRelationship.ClassDM = null;
            //---------------Assert Precondition----------------
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.IsNull(dmRelationship.ClassDM);
            //---------------Execute Test ----------------------
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsNotNull(rule);
            Assert.AreEqual("DMRelationshipSingleToSingleAK", rule.Name);
            Assert.AreEqual(ErrorLevel.Warning, rule.ErrorLevel);
            StringAssert.Contains("One side of the relationship '", rule.Message);
            StringAssert.Contains("'" + dmRelationship + "-" + dmRelationship.ReverseRelationship + "'", rule.Message);
            StringAssert.Contains("' on Classes '" + dmRelationship.ClassDM + "-" + dmRelationship.RelatedClass + "'", rule.Message);
            StringAssert.Contains("' must be a Primary Key while the other must be an Alternate Key.", rule.Message);
        }

//        [Test]
//        public void Test_CreateDMRelationshipSingleToSingleAKRule_WhenRelationshipNull_ShouldRaiseError()
//        {
//            //---------------Set up test pack-------------------
//            //---------------Assert Precondition----------------
//            //---------------Execute Test ----------------------
//            try
//            {
//                new DMRelationshipSingleToSingleAKRule(null);
//                Assert.Fail("expected ArgumentNullException");
//            }
//            //---------------Test Result -----------------------
//            catch (ArgumentNullException ex)
//            {
//                StringAssert.Contains("Value cannot be null", ex.Message);
//                StringAssert.Contains("dmRelationship", ex.ParamName);
//            }
//        }

        [Test]
        public void Test_DMRelationshipPropKeyRule_SingleWhenSingleReverse_BothInAK_ShouldFail()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = CreateClassWithAKAndRelationship();
            DMRelationship dmRelationship = dmClass.OwnerRelationships[0];
            DMClass relatedClass = CreateClassWithAKAndRelationship();
            SetReverseRelationship(dmRelationship, relatedClass.OwnerRelationships[0]);
//            dmRelationship.ReverseRelationship = relatedClass.OwnerRelationships[0];
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.AreEqual(Cardinality.Single, dmRelationship.ReverseRelationship.Cardinality);
            Assert.IsNull(dmClass.ObjectIdentity);
            Assert.IsNull(relatedClass.ObjectIdentity);
            Assert.AreEqual(1, dmClass.UniqueConstraints.Count);
            Assert.AreEqual(1, relatedClass.UniqueConstraints.Count);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsFalse(isValid);
            TestUtilsShared.AssertStringNotEmpty(rule.Message, "rule.Message");
        }

        private static void SetReverseRelationship(DMRelationship dmRelationship, DMRelationship reverseRelationship)
        {
            dmRelationship.Relationships.SetRelatedObject("ReverseRelationship", reverseRelationship);
        }

 
        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_Multiple_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            IDMClass dmClass = dmProperty.Class;
            DMRelationship dmRelationship = new DMRelationship();
            dmRelationship.Cardinality = Cardinality.Multiple;
            dmClass.OwnerRelationships.Add(dmRelationship);
            DMRelationshipProperty relationshipProperty = new DMRelationshipProperty();
            relationshipProperty.Property = dmProperty;
            dmRelationship.RelationshipProperties.Add(relationshipProperty);
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmClass.ObjectIdentity);
            Assert.AreEqual(0, dmClass.UniqueConstraints.Count);
            Assert.AreEqual(Cardinality.Multiple, dmRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenNoReverse_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            IDMClass dmClass = dmProperty.Class;
            DMRelationship dmRelationship = new DMRelationship();
            dmRelationship.Cardinality = Cardinality.Single;
            dmClass.OwnerRelationships.Add(dmRelationship);
            DMRelationshipProperty relationshipProperty = new DMRelationshipProperty();
            relationshipProperty.Property = dmProperty;
            dmRelationship.RelationshipProperties.Add(relationshipProperty);
            SetReverseRelationship(dmRelationship, null);
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmClass.ObjectIdentity);
            Assert.AreEqual(0, dmClass.UniqueConstraints.Count);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.IsNull(dmRelationship.ReverseRelationship);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenMultipleReverse_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            IDMClass dmClass = dmProperty.Class;
            DMRelationship dmRelationship = new DMRelationship();
            dmRelationship.Cardinality = Cardinality.Single;
            dmClass.OwnerRelationships.Add(dmRelationship);
            DMRelationshipProperty relationshipProperty = new DMRelationshipProperty();
            relationshipProperty.Property = dmProperty;
            dmRelationship.RelationshipProperties.Add(relationshipProperty);
            dmRelationship.ReverseRelationship.Cardinality = Cardinality.Multiple;
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmClass.ObjectIdentity);
            Assert.AreEqual(0, dmClass.UniqueConstraints.Count);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.IsNotNull(dmRelationship.ReverseRelationship);
            Assert.AreEqual(Cardinality.Multiple, dmRelationship.ReverseRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenSingleReverse_HasBothPK_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = CreateClassWithPKAndRelationship();
            DMRelationship dmRelationship = dmClass.OwnerRelationships[0];
            DMClass relatedClass = CreateClassWithPKAndRelationship();
            SetReverseRelationship(dmRelationship, relatedClass.OwnerRelationships[0]);
//            dmRelationship.ReverseRelationship = relatedClass.OwnerRelationships[0];
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmClass.ObjectIdentity);
            Assert.IsNotNull(relatedClass.ObjectIdentity);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.AreEqual(Cardinality.Single, dmRelationship.ReverseRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenSingleReverse_HasPKAndRelatedAK_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = CreateClassWithPKAndRelationship();
            DMRelationship dmRelationship = dmClass.OwnerRelationships[0];
            DMClass relatedClass = CreateClassWithAKAndRelationship();
            SetReverseRelationship(dmRelationship, relatedClass.OwnerRelationships[0]);
//            dmRelationship.ReverseRelationship = relatedClass.OwnerRelationships[0];
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmClass.ObjectIdentity);
            Assert.IsNull(relatedClass.ObjectIdentity);
            Assert.AreEqual(1, relatedClass.UniqueConstraints.Count);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.AreEqual(Cardinality.Single, dmRelationship.ReverseRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenSingleReverse_HasAKAndRelatedPK_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = CreateClassWithAKAndRelationship();
            DMRelationship dmRelationship = dmClass.OwnerRelationships[0];
            DMClass relatedClass = CreateClassWithPKAndRelationship();
            SetReverseRelationship(dmRelationship, relatedClass.OwnerRelationships[0]);
//            dmRelationship.ReverseRelationship = relatedClass.OwnerRelationships[0];
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(relatedClass.ObjectIdentity);
            Assert.IsNull(dmClass.ObjectIdentity);
            Assert.AreEqual(1, dmClass.UniqueConstraints.Count);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.AreEqual(Cardinality.Single, dmRelationship.ReverseRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_SingleWhenSingleReverse_NoRelatedClass_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = CreateClassWithPKAndRelationship();
            DMRelationship dmRelationship = dmClass.OwnerRelationships[0];
            SetReverseRelationship(dmRelationship, new DMRelationship());
//            dmRelationship.ReverseRelationship = new DMRelationship();
            dmRelationship.ReverseRelationship.Cardinality = Cardinality.Single;
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmRelationship.ReverseRelationship.ClassDM);
            Assert.IsNotNull(dmClass.ObjectIdentity);
            Assert.AreEqual(Cardinality.Single, dmRelationship.Cardinality);
            Assert.AreEqual(Cardinality.Single, dmRelationship.ReverseRelationship.Cardinality);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMRelationshipSingleToSingleAKRule_WhenNoClass_ShouldPass()
        {
            //---------------Set up test pack-------------------
            DMRelationship dmRelationship = new DMRelationship();
            IBusinessObjectRule rule = new DMRelationshipSingleToSingleAKRule();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmRelationship.ClassDM);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmRelationship);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        private DMClass CreateClassWithAKAndRelationship()
        {
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            IDMClass dmClass = dmProperty.Class;
            DMRelationship dmRelationship = new DMRelationship();
            dmClass.OwnerRelationships.Add(dmRelationship);
            DMRelationshipProperty relationshipProperty = dmRelationship.RelationshipProperties.CreateBusinessObject();
            relationshipProperty.Property = dmProperty;
            DMUniqueConstraint uniqueConstraint = dmClass.UniqueConstraints.CreateBusinessObject();
            uniqueConstraint.AddPropertyInfo(dmProperty);
            return (DMClass) dmClass;
        }

        private DMClass CreateClassWithPKAndRelationship()
        {
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            IDMClass dmClass = dmProperty.Class;
            dmClass.CreateObjectIdentity(dmProperty);
            DMRelationship dmRelationship = new DMRelationship();
            dmClass.OwnerRelationships.Add(dmRelationship);
            DMRelationshipProperty relationshipProperty = dmRelationship.RelationshipProperties.CreateBusinessObject();
            relationshipProperty.Property = dmProperty;
            return (DMClass) dmClass;
        }

        #endregion
    }
}